package com.planet_ink.coffee_mud.core;
import com.planet_ink.coffee_mud.Libraries.interfaces.CatalogLibrary;
import com.planet_ink.coffee_mud.Libraries.interfaces.WorldMap;
import com.planet_ink.coffee_mud.MOBS.interfaces.MOB;
import com.planet_ink.coffee_mud.core.database.DBConnections;
import com.planet_ink.coffee_mud.core.interfaces.MudHost;
import com.planet_ink.miniweb.interfaces.FileManager;

import java.io.*;
import java.nio.charset.Charset;
import java.util.*;
import java.util.regex.Pattern;

/*
Copyright 2000-2013 Bo Zimmerman

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
/**
 * An object to abstractly access the CoffeeMud File System (CMFS), which
 * is a layer of database-based files (VFS) on top of the normal systems
 * file system.  It uses the unix path system.
 * @author Bo Zimmerman
 */
public class CMFile extends File
{
	private static final long serialVersionUID = -3965083655590304708L;

	/** Flag for a CMVFSDir object, allowing you to open it */
	public static final int  VFS_MASK_MASKSAVABLE=1+2+4;
	
	/** Flag for opening a CMFile to suppress error logging */
	public static final int FLAG_LOGERRORS 	= 1;
	/** Flag for opening a CMFile to force allowing a given user to open despite security */
	public static final int FLAG_FORCEALLOW = 2;

	//private static final int VFS_MASK_BINARY=1;
	private static final int VFS_MASK_DIRECTORY=2;
	private static final int VFS_MASK_HIDDEN=4;
	private static final int VFS_MASK_ISLOCAL=8;
	private static final int VFS_MASK_NOWRITEVFS=16;
	private static final int VFS_MASK_NOWRITELOCAL=32;
	private static final int VFS_MASK_NOREADVFS=64;
	private static final int VFS_MASK_NOREADLOCAL=128;
	private static final int VFS_MASK_NODELETEANY=256;

	private static final char pathSeparator=File.separatorChar;
	
	private static final String inCharSet = Charset.defaultCharset().name();
	private static final String outCharSet = Charset.defaultCharset().name();
	
	private static CMVFSDir[] vfs=new CMVFSDir[256];
	
	private static CatalogLibrary catalogPluginAdded=null;
	private static WorldMap mapPluginAdded=null;
	
	private boolean logErrors=false;

	private int vfsBits=0;
	private String localPath=null;
	private String path=null;
	private String name=null;
	private String author=null;
	private MOB accessor=null;
	private long modifiedDateTime=System.currentTimeMillis();
	private File localFile=null;
	private boolean demandVFS=false;
	private boolean demandLocal=false;
	private String parentDir=null;

	/**
	 * Constructor for a CMFS file from a full path. The creating/opening
	 * user may be null to ignore user security features.  Will always
	 * suppress error logging, and if a user is given, they must have
	 * permission to open/create the file.
	 * @param absolutePath the path to the file to open/create
	 * @param user the user to check for permissions on
	 */
	public CMFile (final String absolutePath, final MOB user)
	{
		this(null, absolutePath,user,0);
	}
	
	/**
	 * Constructor for a CMFS file from a full path. The creating/opening
	 * user may be null to ignore user security features.
	 * @param absolutePath the path to the file to open/create
	 * @param user the user to check for permissions on
	 * @param flagBitmap bitmap flag to turn on error logging or force allow
	 */
	public CMFile (String absolutePath, final MOB user, final int flagBitmap)
	{ 
		this(null, absolutePath,user,flagBitmap);
	}
	
	private CMFile (CMVFSFile info, final String absolutePath, final MOB user)
	{
		this(info, absolutePath,user,0);
	}
	
	private CMFile (CMVFSFile info, String absolutePath, final MOB user, final int flagBitmap)
	{ 
		super(parsePathParts(absolutePath)[2]);
		boolean pleaseLogErrors=(flagBitmap&FLAG_LOGERRORS)>0;
		boolean forceAllow=(flagBitmap&FLAG_FORCEALLOW)>0;
		accessor=user;
		localFile=null;
		logErrors=pleaseLogErrors;
		demandLocal=absolutePath.trim().startsWith("//");
		demandVFS=absolutePath.trim().startsWith("::");
		if(accessor!=null) author=accessor.Name();
		final String[] pathParts=parsePathParts(absolutePath);
		absolutePath=pathParts[0];
		path=pathParts[1];
		name=pathParts[2];
		parentDir=path;
		localPath=path.replace('/',pathSeparator);
		// fill in all we can
		vfsBits=0;
		if(info==null)
			info=getVFSInfo(absolutePath);
		String ioPath=getIOReadableLocalPathAndName();
		localFile=new File(ioPath);
		if(!localFile.exists())
		{
			File localDir=new File(".");
			int endZ=-1;
			boolean found=true;
			if((localDir.exists())&&(localDir.isDirectory()))
				parentDir="//"+localPath;
			while((!localFile.exists())&&(endZ<ioPath.length())&&(localDir.exists())&&(localDir.isDirectory())&&(found))
			{
				int startZ=endZ+1;
				endZ=ioPath.indexOf(pathSeparator,startZ);
				if(endZ<0)
					endZ=ioPath.length();
				String[] files=localDir.list();
				found=false;
				for(int f=0;f<files.length;f++)
				{
					if(files[f].equalsIgnoreCase(ioPath.substring(startZ,endZ)))
					{
						if(!files[f].equals(ioPath.substring(startZ,endZ)))
							ioPath=ioPath.substring(0,startZ)+files[f]+((endZ<ioPath.length())?ioPath.substring(endZ):"");
						found=true;
						break;
					}
				}
				if(found)
				{
					if(endZ==ioPath.length())
					{
						int lastSep=ioPath.lastIndexOf(pathSeparator);
						if(lastSep>=0)
						{
							localPath=ioPath.substring(0,lastSep);
							name=ioPath.substring(lastSep+1);
						}
						else
							name=ioPath;
						localFile=new File(getIOReadableLocalPathAndName());
					}
					else
						localDir=new File(localDir.getAbsolutePath()+pathSeparator+ioPath.substring(startZ,endZ));
				}
			}
		}
		else
			parentDir="::"+parentDir;

		if((info!=null)&&((!demandLocal)||(!localFile.exists())))
		{
			vfsBits=vfsBits|info.getMaskBits(accessor);
			author=info.author;
			modifiedDateTime=info.modifiedDateTime;
		}
		else
		{
			modifiedDateTime=localFile.lastModified();
			if(localFile.isHidden()) vfsBits=vfsBits|CMFile.VFS_MASK_HIDDEN;
		}

		boolean doesFilenameExistInVFS=(info!=null) && (info.path.equalsIgnoreCase(absolutePath) || info.path.equalsIgnoreCase(absolutePath+"/"));
		boolean doesExistAsPathInVFS= (info!=null) && doesFilenameExistInVFS && ((info instanceof CMVFSDir)||(CMath.bset(info.mask, CMFile.VFS_MASK_DIRECTORY)));
		if((info!=null)&&(!doesFilenameExistInVFS))
			Log.debugOut(new Exception("CMFile sent: '"+absolutePath+"' != vfs found:'"+info.path+"' fetch/create error."));

		boolean isADirectory=((localFile!=null)&&(localFile.exists())&&(localFile.isDirectory()))
						   ||doesExistAsPathInVFS;
		boolean allowedToTraverseAsDirectory=isADirectory
										   &&((accessor==null)||CMSecurity.canTraverseDir(accessor,accessor.location(),absolutePath));
		boolean allowedToWriteVFS=((forceAllow)||(accessor==null)||(CMSecurity.canAccessFile(accessor,accessor.location(),absolutePath,true)));
		boolean allowedToReadVFS=(doesFilenameExistInVFS&&allowedToWriteVFS)||allowedToTraverseAsDirectory;
		boolean allowedToWriteLocal=((forceAllow)||(accessor==null)||(CMSecurity.canAccessFile(accessor,accessor.location(),absolutePath,false)));
		boolean allowedToReadLocal=(localFile!=null)
									&&(localFile.exists())
									&&(allowedToWriteLocal||allowedToTraverseAsDirectory);

		if(!allowedToReadVFS) vfsBits=vfsBits|CMFile.VFS_MASK_NOREADVFS;

		if(!allowedToReadLocal) vfsBits=vfsBits|CMFile.VFS_MASK_NOREADLOCAL;

		if(!allowedToWriteVFS) vfsBits=vfsBits|CMFile.VFS_MASK_NOWRITEVFS;

		if(!allowedToWriteLocal) vfsBits=vfsBits|CMFile.VFS_MASK_NOWRITELOCAL;

		if(allowedToTraverseAsDirectory)  vfsBits=vfsBits|CMFile.VFS_MASK_DIRECTORY;

		if((demandVFS)&&(!allowedToReadVFS))  vfsBits=vfsBits|CMFile.VFS_MASK_NOREADLOCAL;
		if((demandVFS)&&(!allowedToWriteVFS))  vfsBits=vfsBits|CMFile.VFS_MASK_NOWRITELOCAL;

		if((demandLocal)&&(!allowedToReadLocal))  vfsBits=vfsBits|CMFile.VFS_MASK_NOREADVFS;
		if((demandLocal)&&(!allowedToWriteLocal))  vfsBits=vfsBits|CMFile.VFS_MASK_NOWRITEVFS;

		if((!demandVFS)
		&&(demandLocal||((!allowedToReadVFS) && (allowedToWriteLocal))))
			vfsBits=vfsBits|CMFile.VFS_MASK_ISLOCAL;
	}

	/**
	 * 
	 * @author Bo Zimmerman
	 *
	 */
	public static class CMVFSFile 
	{
		public String fName;
		public String uName;
		public String path;
		public int mask;
		public long modifiedDateTime;
		public String author;
		public Object data = null;
		
		public CMVFSFile(final String path, final int mask, final long modifiedDateTime, final String author) 
		{
			this.path=path;
			this.fName=path;
			int x=path.lastIndexOf('/');
			if(x==path.length()-1) x=path.lastIndexOf('/',path.length()-2);
			if(x>=0) this.fName=path.substring(x+1);
			while(this.fName.startsWith("/")) this.fName=this.fName.substring(1);
			while(this.fName.endsWith("/")) this.fName=this.fName.substring(0,this.fName.length()-1);
			this.uName=this.fName.toUpperCase();
			this.mask=mask;
			this.modifiedDateTime=modifiedDateTime;
			this.author=author;
		}
		
		public void copyInto(final CMVFSFile f2)
		{
			f2.author=author;
			f2.data=data;
			f2.fName=fName;
			f2.mask=mask;
			f2.modifiedDateTime=modifiedDateTime;
			f2.path=path;
			f2.uName=uName;
			if((this instanceof CMVFSDir)&&(f2 instanceof CMVFSDir))
			{
				((CMVFSDir)f2).parent=((CMVFSDir)this).parent;
				if((((CMVFSDir)this).files!=null)
				&&(((CMVFSDir)this).files.length>0)
				&&(((CMVFSDir)f2).files==null))
					((CMVFSDir)f2).files=((CMVFSDir)this).files;
			}
		}
		
		public int getMaskBits(MOB accessor)
		{
			return mask;
		}
		
		public Object readData()
		{
			CMVFSFile f=CMLib.database().DBReadVFSFile(path);
			if(f!=null)
				return f.data;
			return null;
		}
		public void saveData(String filename, int vfsBits, String author, Object O)
		{
			CMVFSFile info = new CMVFSFile(filename,vfsBits&VFS_MASK_MASKSAVABLE,System.currentTimeMillis(),author);
			getVFSDirectory().add(info);
			CMLib.database().DBUpSertVFSFile(filename,vfsBits,author,System.currentTimeMillis(),O);
		}
	}
	
	public static class CMVFSDir extends CMVFSFile
	{
		protected CMVFSFile[] files=null;
		public CMVFSDir parent=null;
		
		public static Comparator<CMVFSFile> fcomparator=new Comparator<CMVFSFile>() {
			public int compare(final CMVFSFile arg0, final CMVFSFile arg1) { return arg0.uName.compareTo(arg1.uName); }
		};
		
		protected CMVFSFile[] getFiles() { return files; }
		
		public CMVFSDir(final CMVFSDir parent, final String path)
		{
			super(path,VFS_MASK_DIRECTORY,System.currentTimeMillis(),"SYS");
			this.parent=parent;
		}

		public CMVFSDir(final CMVFSDir parent, final int mask, final String path)
		{
			super(path,mask|VFS_MASK_DIRECTORY,System.currentTimeMillis(),"SYS");
			this.parent=parent;
		}

		private CMVFSDir(final CMVFSDir parent, final String path, final int mask, final long modifiedDateTime, final String author)
		{
			super(path,mask,modifiedDateTime,author);
			this.parent=parent;
		}

		public synchronized CMVFSDir fetchSubDir(final String path, final boolean create)
		{
			final String[] ppath=path.split("/");
			CMVFSDir currDir=this;
			for(final String p : ppath)
				if(p.length()>0)
				{
					synchronized(currDir)
					{
						final CMVFSDir key = new CMVFSDir(currDir,currDir.path+p+"/");
						int dex = -1;
						CMVFSFile[] files=currDir.getFiles();
						if(files!=null)
							dex=Arrays.binarySearch(files, key, fcomparator);
						if((files!=null)&&(dex>=0))
						{
							if(files[dex] instanceof CMVFSDir)
							{
								currDir=(CMVFSDir)files[dex];
								continue;
							}
							return null; // found a step, but its a file, not a subdir
						}
						else
						if(!create)
							return null;
						if(files==null) currDir.files=new CMVFSFile[0];
						currDir.files=Arrays.copyOf(currDir.files, currDir.files.length+1);
						currDir.files[currDir.files.length-1]=key;
						Arrays.sort(currDir.files,fcomparator);
						currDir=key;
					}
				}
			return currDir;
		}

		public final boolean add(CMVFSFile f)
		{
			int x=f.path.lastIndexOf('/');
			if(x==f.path.length()-1)
				x=f.path.lastIndexOf('/',x-1);
			CMVFSDir subDir=this;
			if((x>0)&&(!f.path.substring(0,x+1).equals(path)))
				subDir=fetchSubDir(f.path.substring(0,x),true);
			if(subDir!=null)
			synchronized(subDir)
			{
				if(subDir.files==null) subDir.files=new CMVFSFile[0];
				final CMVFSFile old = subDir.get(f);
				if(old!=null)
					f.copyInto(old);
				else
				{
					subDir.files=Arrays.copyOf(subDir.files, subDir.files.length+1);
					if(CMath.bset(f.mask, CMFile.VFS_MASK_DIRECTORY))
					{
						if(!(f instanceof CMVFSDir))
						{
							CMVFSDir d = new CMVFSDir(subDir,f.path,f.mask,f.modifiedDateTime,f.author);
							f.copyInto(d);
							f=d;
						}
					}
					subDir.files[subDir.files.length-1]=f;
				}
				Arrays.sort(subDir.files,fcomparator);
			}
			else
				return false;
			return true;
		}
		
		public final boolean delete(final CMVFSFile file)
		{
			final CMVFSDir dir = vfsV();
			if(dir==null) return false;
			int x=file.path.lastIndexOf('/');
			CMVFSDir subDir=dir;
			if(x>0)
				subDir=dir.fetchSubDir(file.path.substring(0,x),true);
			synchronized(subDir)
			{
				if((x==file.path.length()-1)&&(subDir.parent!=null))
				{
					if(subDir.parent.files!=null)
					{
						final List<CMVFSFile> list = new Vector<CMVFSFile>();
						list.addAll(Arrays.asList(subDir.parent.files));
						if(!list.remove(subDir))
							return false;
						subDir.parent.files = list.toArray(new CMVFSFile[0]);
						return true;
					}
					else
						return false;
				}
				else
				{
					final List<CMVFSFile> list = new Vector<CMVFSFile>();
					if(subDir.files==null)
						return false;
					list.addAll(Arrays.asList(subDir.files));
					if(!list.remove(file))
						return false;
					subDir.files = list.toArray(new CMVFSFile[0]);
					return true;
				}
			}
		}
		
		private final synchronized CMVFSFile get(final String fileName)
		{
			CMVFSFile[] files=getFiles();
			if(files==null) return null;
			final CMVFSFile key = new CMVFSFile(fileName, 0, 0, "");
			int dex = Arrays.binarySearch(files, key, fcomparator);
			if(dex>=0) return files[dex];
			return null;
		}
		
		private final synchronized CMVFSFile get(final CMVFSFile file)
		{
			CMVFSFile[] files=getFiles();
			if(files==null) return null;
			int dex = Arrays.binarySearch(files, file, fcomparator);
			if(dex>=0) return file;
			return null;
		}
		
		public final synchronized CMVFSFile fetch(final String filePath)
		{
			final int x=filePath.lastIndexOf('/');
			CMVFSDir dir = this;
			if(x>=0)
				dir = this.fetchSubDir(filePath.substring(0,x), false);
			if(dir==null) return null;
			if(x==filePath.length()-1) return dir;
			String fileName=(x<0)?filePath:filePath.substring(x+1).trim();
			if(fileName.length()==0)
				return dir;
			return dir.get(fileName);
		}
	}
	
	private static final CMVFSDir vfsV() 
	{
		return vfs[Thread.currentThread().getThreadGroup().getName().charAt(0)];
	}
	
	@Override
	public final CMFile getParentFile(){return new CMFile(path,accessor);}

	public final boolean mustOverwrite()
	{
		if(!isDirectory())
			return canRead();
		if(isLocalFile())
			return ((localFile!=null)&&(localFile.isDirectory()));
		String filename=getVFSPathAndName();
		CMVFSFile info=getVFSInfo(filename);
		if(!filename.endsWith("/")) filename=filename+"/";
		if(info==null) info=getVFSInfo(filename);
		return (info!=null);
	}

	@Override
	public final boolean canExecute()
	{
		return false;
	}
	@Override
	public final boolean canRead()
	{
		if(!exists()) return false;
		if(CMath.bset(vfsBits,CMFile.VFS_MASK_ISLOCAL))
		{
			if(CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADLOCAL))
				return false;
		}
		else
		{
			if(CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADVFS))
				return false;
		}
		return true;
	}
	@Override
	public final boolean canWrite()
	{
		if(CMath.bset(vfsBits,CMFile.VFS_MASK_ISLOCAL))
		{
			if(CMath.bset(vfsBits,CMFile.VFS_MASK_NOWRITELOCAL))
				return false;
		}
		else
		{
			if(CMath.bset(vfsBits,CMFile.VFS_MASK_NOWRITEVFS))
				return false;
		}
		return true;
	}

	public final boolean demandedVFS(){return demandVFS;}
	public final boolean demandedLocal(){return demandLocal;}
	@Override
	public final boolean isDirectory(){return exists()&&CMath.bset(vfsBits,CMFile.VFS_MASK_DIRECTORY);}
	@Override
	public final boolean exists(){ return !(CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADVFS)&&CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADLOCAL));}
	@Override
	public final boolean isFile(){return canRead()&&(!CMath.bset(vfsBits,CMFile.VFS_MASK_DIRECTORY));}
	@Override
	public final long lastModified(){return modifiedDateTime;}
	public final String author(){return ((author!=null))?author:"SYS_UNK";}
	public final boolean isLocalFile(){return CMath.bset(vfsBits,CMFile.VFS_MASK_ISLOCAL);}
	public final boolean isVFSFile(){return (!CMath.bset(vfsBits,CMFile.VFS_MASK_ISLOCAL));}
	public final boolean canVFSEquiv(){return (!CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADVFS));}
	public final boolean canLocalEquiv(){return (!CMath.bset(vfsBits,CMFile.VFS_MASK_NOREADLOCAL));}
	@Override
	public final String getName(){return name;}
	@Override
	public final String getAbsolutePath(){return "/"+getVFSPathAndName();}
	@Override
	public final String getCanonicalPath(){return getVFSPathAndName();}
	public final String getLocalPathAndName()
	{
		if(path.length()==0)
			return name;
		return localPath+pathSeparator+name;
	}
	public final String getIOReadableLocalPathAndName()
	{
		final String s=getLocalPathAndName();
		if(s.trim().length()==0) return ".";
		return s;
	}
	public final String getVFSPathAndName()
	{
		if(path.length()==0)
			return name;
		return path+'/'+name;
	}

	public final boolean mayDeleteIfDirectory()
	{
		if(!isDirectory()) return true;
		if(localFile!=null)
		{
			if(!localFile.isDirectory())
				return false;
			if(localFile.list().length>0)
				return false;
		}
		if(getVFSDirectory().fetchSubDir(getVFSPathAndName().toUpperCase(), false)!=null)
			return false;
		return true;
	}

	public final boolean deleteLocal()
	{
		if(!exists()) return false;
		if(!canWrite()) return false;
		if(CMath.bset(vfsBits, CMFile.VFS_MASK_NODELETEANY))
			return false;
		if(!mayDeleteIfDirectory()) return false;
		if((canLocalEquiv())&&(localFile!=null))
			return localFile.delete();
		return false;
	}
	public final boolean deleteVFS()
	{
		if(!exists()) return false;
		if(!canWrite()) return false;
		if(CMath.bset(vfsBits, CMFile.VFS_MASK_NODELETEANY))
			return false;
		if(!mayDeleteIfDirectory()) return false;
		if(canVFSEquiv())
		{
			String name=getVFSPathAndName();
			if(isDirectory()) name=name+"/";
			final CMVFSFile info=getVFSInfo(name);
			if(info==null)
				return false;
			CMLib.database().DBDeleteVFSFile(info.path);
			getVFSDirectory().delete(info);
			return true;
		}
		return false;
	}

	public final boolean deleteAll()
	{
		if(!exists()) return false;
		if(!canWrite()) return false;
		if(CMath.bset(vfsBits, CMFile.VFS_MASK_NODELETEANY))
			return false;
		if(!mayDeleteIfDirectory()) return false;
		if(demandVFS)
			return deleteVFS();
		if(demandLocal)
			return deleteLocal();
		boolean delVfs=deleteVFS();
		boolean delLoc=deleteLocal();
		return delVfs || delLoc;
	}
	
	@Override
	public final boolean delete()
	{
		if(!exists()) return false;
		if(!canWrite()) return false;
		if(CMath.bset(vfsBits, CMFile.VFS_MASK_NODELETEANY))
			return false;
		if(!mayDeleteIfDirectory()) return false;
		if((isVFSDirectory())&&(!demandLocal))
			return deleteVFS();
		if((isLocalDirectory())&&(!demandVFS))
			return deleteLocal();
		if(isVFSFile())
			return deleteVFS();
		if(isLocalFile())
			return deleteLocal();
		return false;
	}

	public final InputStream getTextStream()
	{
		return new ByteArrayInputStream(text().toString().getBytes());
	}
	
	public final StringBuffer text()
	{
		final StringBuffer buf=new StringBuffer("");
		if(!canRead())
		{
			if(logErrors)
				Log.errOut("CMFile","Access error on file '"+getVFSPathAndName()+"'.");
			return buf;
		}
		if(isVFSFile())
		{
			final CMVFSFile info=getVFSInfo(getVFSPathAndName());
			if(info!=null)
			{
				final Object data=getVFSData(getVFSPathAndName());
				if(data==null) return buf;
				if(data instanceof String)
					return new StringBuffer((String)data);
				if(data instanceof StringBuffer)
					return (StringBuffer)data;
				if(data instanceof StringBuilder)
					return new StringBuffer((StringBuilder)data);
				if(data instanceof byte[])
					return new StringBuffer(CMStrings.bytesToStr((byte[])data));
			}
			else
			if(logErrors)
				Log.errOut("CMFile","VSF File '"+getVFSPathAndName()+"' not found.");
			return buf;
		}

		BufferedReader reader = null;
		try
		{
			String charSet=CMProps.getVar(CMProps.Str.CHARSETINPUT);
			if((charSet==null)||(charSet.length()==0))
				charSet=inCharSet;
			reader=new BufferedReader(
				   new InputStreamReader(
				   new FileInputStream(
					   getIOReadableLocalPathAndName()
			),charSet));
			String line="";
			while((line!=null)&&(reader.ready()))
			{
				line=reader.readLine();
				if(line!=null)
				{
					buf.append(line);
					buf.append("\n\r");
				}
			}
		}
		catch(Exception e)
		{
			if(logErrors)
				Log.errOut("CMFile",e.getMessage());
			return buf;
		}
		finally
		{
			try
			{
				if ( reader != null )
				{
					reader.close();
					reader = null;
				}
			}
			catch ( final IOException ignore )
			{

			}
		}
		return buf;
	}

	public final InputStream getUnformattedTextStream()
	{
		return new ByteArrayInputStream(text().toString().getBytes());
	}
	
	public final StringBuffer textUnformatted()
	{
		final StringBuffer buf=new StringBuffer("");
		if(!canRead())
		{
			if(logErrors)
				Log.errOut("CMFile","Access error on file '"+getVFSPathAndName()+"'.");
			return buf;
		}
		if(isVFSFile())
		{
			final CMVFSFile info=getVFSInfo(getVFSPathAndName());
			if(info!=null)
			{
				final Object data=getVFSData(getVFSPathAndName());
				if(data==null) return buf;
				if(data instanceof String)
					return new StringBuffer((String)data);
				if(data instanceof StringBuffer)
					return (StringBuffer)data;
				if(data instanceof StringBuilder)
					return new StringBuffer((StringBuilder)data);
				if(data instanceof byte[])
					return new StringBuffer(CMStrings.bytesToStr((byte[])data));
			}
			else
			if(logErrors)
				Log.errOut("CMFile","VSF File '"+getVFSPathAndName()+"' not found.");
			return buf;
		}

		Reader F = null;
		try
		{
			String charSet=CMProps.getVar(CMProps.Str.CHARSETINPUT);
			if((charSet==null)||(charSet.length()==0))
				charSet=inCharSet;
			F=new InputStreamReader(
			   new FileInputStream(
				   getIOReadableLocalPathAndName()
			 ),charSet);
			char c=' ';
			while(F.ready())
			{
				c=(char)F.read();
				if(c<0) break;
				buf.append(c);
			}
		}
		catch(Exception e)
		{
			if(logErrors)
				Log.errOut("CMFile",e.getMessage());
		}
		finally
		{
			try
			{
				if ( F != null )
				{
					F.close();
					F = null;
				}
			}
			catch ( final IOException ignore )
			{

			}
		}
		return buf;
	}

	public final InputStream getRawStream()
	{
		return new ByteArrayInputStream(raw());
	}
	
	public final byte[] raw()
	{
		byte[] buf=new byte[0];
		if(!canRead())
		{
			if(logErrors)
				Log.errOut("CMFile","Access error on file '"+getVFSPathAndName()+"'.");
			return buf;
		}
		if(isVFSFile())
		{
			CMVFSFile info=getVFSInfo(getVFSPathAndName());
			if(info!=null)
			{
				Object data=getVFSData(getVFSPathAndName());
				if(data==null) return buf;
				if(data instanceof byte[])
					return (byte[])data;
				if(data instanceof String)
					return CMStrings.strToBytes((String)data);
				if(data instanceof StringBuffer)
					return CMStrings.strToBytes(((StringBuffer)data).toString());
				if(data instanceof StringBuilder)
					return CMStrings.strToBytes(((StringBuilder)data).toString());
			}
			else
			if(logErrors)
				Log.errOut("CMFile","VSF File '"+getVFSPathAndName()+"' not found.");
			return buf;
		}

		DataInputStream fileIn = null;
		try
		{
			fileIn = new DataInputStream( new BufferedInputStream( new FileInputStream(getIOReadableLocalPathAndName()) ) );
			buf = new byte [ fileIn.available() ];
			fileIn.readFully(buf);
			fileIn.close();
		}
		catch(Exception e)
		{
			if(logErrors)
				Log.errOut("CMFile",e.getMessage());
		}
		finally
		{
			try
			{
				if ( fileIn != null )
				{
					fileIn.close();
					fileIn = null;
				}
			}
			catch ( final IOException ignore )
			{

			}
		}
		return buf;
	}

	public final StringBuffer textVersion(byte[] bytes)
	{
		final StringBuffer text=new StringBuffer(CMStrings.bytesToStr(bytes));
		for(int i=0;i<text.length();i++)
			if((text.charAt(i)<0)||(text.charAt(i)>127))
				return null;
		return text;
	}

	public boolean saveRaw(Object data)
	{
		if(data==null)
		{
			Log.errOut("CMFile","Unable to save file '"+getVFSPathAndName()+"': No Data.");
			return false;
		}
		if((CMath.bset(vfsBits,CMFile.VFS_MASK_DIRECTORY))
		||(!canWrite()))
		{
			Log.errOut("CMFile","Access error saving file '"+getVFSPathAndName()+"'.");
			return false;
		}

		Object O=null;
		if(data instanceof String)
			O=new StringBuffer((String)data);
		else
		if(data instanceof StringBuffer)
			O=data;
		else
		if(data instanceof StringBuilder)
			O=new StringBuffer((StringBuilder)data);
		else
		if(data instanceof byte[])
		{
			StringBuffer test=textVersion((byte[])data);
			if(test!=null)
				O=test;
			else
				O=data;
		}
		else
			O=new StringBuffer(data.toString());
		if(!isLocalFile())
		{
			String filename=getVFSPathAndName();
			CMVFSFile info=getVFSInfo(filename);
			if(info!=null)
			{
				filename=info.path;
				getVFSDirectory().delete(info);
			}
			if(vfsBits<0) vfsBits=0;
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADVFS);
			if(info!=null)
				info.saveData(filename,vfsBits,author(),O);
			else
			{
				info = new CMVFSFile(filename,vfsBits&VFS_MASK_MASKSAVABLE,System.currentTimeMillis(),author);
				getVFSDirectory().add(info);
				CMLib.database().DBUpSertVFSFile(filename,vfsBits,author,System.currentTimeMillis(),O);
			}
			return true;
		}

		FileOutputStream FW = null;
		try
		{
			File F=new File(getIOReadableLocalPathAndName());
			if(O instanceof StringBuffer)
				O=CMStrings.strToBytes(((StringBuffer)O).toString());
			if(O instanceof StringBuilder)
				O=CMStrings.strToBytes(((StringBuilder)data).toString());
			if(O instanceof String)
				O=CMStrings.strToBytes(((String)O));
			if(O instanceof byte[])
			{
				FW=new FileOutputStream(F,false);
				FW.write((byte[])O);
				FW.flush();
				FW.close();
			}
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADLOCAL);
			return true;
		}
		catch(IOException e)
		{
			Log.errOut("CMFile","Error Saving '"+getIOReadableLocalPathAndName()+"': "+e.getMessage());
		}
		finally
		{
			try
			{
				if ( FW != null )
				{
					FW.close();
					FW = null;
				}
			}
			catch ( final IOException ignore )
			{

			}
		}
		return false;
	}

	public boolean saveText(Object data){ return saveText(data,false);}
	public boolean saveText(Object data, boolean append)
	{
		if(data==null)
		{
			Log.errOut("CMFile","Unable to save file '"+getVFSPathAndName()+"': No Data.");
			return false;
		}
		if((CMath.bset(vfsBits,CMFile.VFS_MASK_DIRECTORY))
		||(!canWrite()))
		{
			Log.errOut("CMFile","Access error saving file '"+getVFSPathAndName()+"'.");
			return false;
		}

		StringBuffer O=null;
		if(data instanceof String)
			O=new StringBuffer((String)data);
		else
		if(data instanceof StringBuffer)
			O=(StringBuffer)data;
		else
		if(data instanceof StringBuilder)
			O=new StringBuffer((StringBuilder)data);
		else
		if(data instanceof byte[])
			O=new StringBuffer(CMStrings.bytesToStr((byte[])data));
		else
			O=new StringBuffer(data.toString());
		if(!isLocalFile())
		{
			if(append) O=new StringBuffer(text().append(O).toString());
			String filename=getVFSPathAndName();
			CMVFSFile info=getVFSInfo(filename);
			if(info!=null)
			{
				filename=info.path;
				if(vfsV()!=null) vfsV().delete(info);
			}
			if(vfsBits<0) vfsBits=0;
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADVFS);
			if(info!=null)
				info.saveData(filename,vfsBits,author(),O);
			else
			{
				info = new CMVFSFile(filename,vfsBits&VFS_MASK_MASKSAVABLE,System.currentTimeMillis(),author);
				getVFSDirectory().add(info);
				CMLib.database().DBUpSertVFSFile(filename,vfsBits,author,System.currentTimeMillis(),O);
			}
			return true;
		}

		Writer FW = null;
		try
		{
			String charSet=CMProps.getVar(CMProps.Str.CHARSETOUTPUT);
			if((charSet==null)||(charSet.length()==0))
				charSet=outCharSet;
			File F=new File(getIOReadableLocalPathAndName());
			FW=new OutputStreamWriter(new FileOutputStream(F,append),charSet);
			FW.write(saveBufNormalize(O).toString());
			FW.close();
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADLOCAL);
			return true;
		}
		catch(IOException e)
		{
			Log.errOut("CMFile","Error Saving '"+getIOReadableLocalPathAndName()+"': "+e.getMessage());
		}
		finally
		{
			try
			{
				if ( FW != null )
				{
					FW.close();
					FW = null;
				}
			}
			catch ( final IOException ignore )
			{

			}
		}
		return false;
	}

	@Override
	public final boolean mkdir()
	{
		if(mustOverwrite())
		{
			Log.errOut("CMFile","File exists '"+getVFSPathAndName()+"'.");
			return false;
		}
		if(!canWrite())
		{
			Log.errOut("CMFile","Access error making directory '"+getVFSPathAndName()+"'.");
			return false;
		}
		if(!isLocalFile())
		{
			String filename=getVFSPathAndName();
			CMVFSFile info=getVFSInfo(filename);
			if(!filename.endsWith("/")) filename=filename+"/";
			if(info==null) info=getVFSInfo(filename);
			if(info!=null)
			{
				Log.errOut("CMFile","File exists '"+getVFSPathAndName()+"'.");
				return false;
			}
			if(vfsBits<0) vfsBits=0;
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADVFS);
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOWRITEVFS);
			vfsBits=vfsBits|CMFile.VFS_MASK_DIRECTORY;
			info=new CMVFSFile(filename,vfsBits&VFS_MASK_MASKSAVABLE,System.currentTimeMillis(),author());
			vfsV().add(info);
			CMLib.database().DBUpSertVFSFile(filename,vfsBits,author(),System.currentTimeMillis(),new StringBuffer(""));
			return true;
		}
		String fullPath=getIOReadableLocalPathAndName();
		File F=new File(fullPath);
		File PF=F.getParentFile();
		Vector<File> parents=new Vector<File>();
		while(PF!=null)
		{
			parents.addElement(PF);
			PF=PF.getParentFile();
		}
		for(int p=parents.size()-1;p>=0;p--)
		{
			PF=parents.elementAt(p);
			if((PF.exists())&&(PF.isDirectory())) continue;
			if((PF.exists()&&(!PF.isDirectory()))||(!PF.mkdir()))
			{
				Log.errOut("CMFile","Unable to mkdir '"+PF.getAbsolutePath()+"'.");
				return false;
			}
		}
		if(F.exists())
		{
			Log.errOut("CMFile","File exists '"+fullPath+"'.");
			return false;
		}
		if(F.mkdir())
		{
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOREADLOCAL);
			vfsBits=vfsBits|CMFile.VFS_MASK_DIRECTORY;
			vfsBits=CMath.unsetb(vfsBits,CMFile.VFS_MASK_NOWRITELOCAL);
			return true;
		}
		return false;
	}

	@Override
	public final String[] list()
	{
		if(!isDirectory()) return new String[0];
		final CMFile[] CF=listFiles();
		final String[] list=new String[CF.length];
		for(int f=0;f<CF.length;f++)
			list[f]=CF[f].getName();
		return list;
	}

	private static final CMVFSFile getVFSInfo(final String filename)
	{
		final CMVFSDir vfs=getVFSDirectory();
		if(vfs==null) return null;
		final String vfsFilename=vfsifyFilename(filename);
		return vfs.fetch(vfsFilename);
	}

	private static final Object getVFSData(final String filename)
	{
		final CMVFSFile file=getVFSInfo(filename);
		if(file!=null)
			return file.readData();
		return null;
	}

	public final boolean isVFSDirectory()
	{
		String dir=getVFSPathAndName().toLowerCase();
		if(!dir.endsWith("/")) dir+="/";
		final CMVFSFile file=getVFSInfo(dir);
		return (file instanceof CMVFSDir);
	}
	
	public final boolean isLocalDirectory()
	{
		return (localFile!=null)&&(localFile.isDirectory());
	}

	@Override
	public final CMFile[] listFiles()
	{
		if((!isDirectory())||(!canRead()))
			return new CMFile[0];
		final String prefix=demandLocal?"//":(demandVFS?"::":"");
		final Vector<CMFile> dir=new Vector<CMFile>();
		final Vector<String> fcheck=new Vector<String>();
		String thisDir=getVFSPathAndName();
		CMVFSDir vfs=getVFSDirectory();
		String vfsSrchDir=thisDir+"/";
		if(thisDir.length()==0) vfsSrchDir="";
		if(!demandLocal)
		{
			if(vfsSrchDir.length()>0)
				vfs=vfs.fetchSubDir(vfsSrchDir, false);
			CMVFSFile[] vfsFiles=(vfs!=null)?vfs.getFiles():null;
			if((vfsFiles!=null)&&(vfsFiles.length>0))
				for(CMVFSFile file : vfsFiles)
				{
					CMFile CF=new CMFile(file,prefix+file.path,accessor);
					if((CF.canRead())
					&&(!fcheck.contains(file.uName)))
					{
						fcheck.addElement(file.uName);
						dir.addElement(CF);
					}
				}
		}

		if(!demandVFS)
		{
			thisDir=getIOReadableLocalPathAndName();
			final File F=new File(thisDir);
			if(F.isDirectory())
			{
				final File[] list=F.listFiles();
				File F2=null;
				for(int l=0;l<list.length;l++)
				{
					F2=list[l];
					String thisPath=vfsifyFilename(thisDir)+"/"+F2.getName();
					String thisName=F2.getName();
					CMFile CF=new CMFile(prefix+thisPath,accessor);
					if((CF.canRead())
					&&(!fcheck.contains(thisName.toUpperCase())))
						dir.addElement(CF);
				}
			}
		}
		final CMFile[] finalDir=new CMFile[dir.size()];
		for(int f=0;f<dir.size();f++)
			finalDir[f]=dir.elementAt(f);
		return finalDir;
	}

	public static final CMVFSDir getVFSDirectory()
	{
		CMVFSDir vvfs=vfsV();
		if(vvfs==null)
		{
			if(CMLib.database()==null) return new CMVFSDir(null,"",CMFile.VFS_MASK_DIRECTORY,System.currentTimeMillis(),"SYS");
			char threadCode=Thread.currentThread().getThreadGroup().getName().charAt(0);
			if(threadCode==MudHost.MAIN_HOST)
				vvfs=vfs[threadCode]=CMLib.database().DBReadVFSDirectory();
			else
			{
				if(CMProps.isPrivateToMe("DBVFS"))
					vvfs=vfs[threadCode]=CMLib.database().DBReadVFSDirectory();
				else
				if(vfs[MudHost.MAIN_HOST]!=null)
				{
					vfs[threadCode]=vfs[MudHost.MAIN_HOST];
					return vfs[threadCode];
				}
				else
					vvfs=vfs[threadCode]=vfs[MudHost.MAIN_HOST]=CMLib.database().DBReadVFSDirectory();
			}
		}
		if((catalogPluginAdded!=CMLib.catalog())&&(CMLib.catalog()!=null))
		{
			catalogPluginAdded=CMLib.catalog();
			CMVFSDir dir=vvfs.fetchSubDir("/resources/catalog", false);
			if(dir!=null) vvfs.delete(dir);
			dir=vvfs.fetchSubDir("/resources", true);
			vvfs.add(CMLib.catalog().getCatalogRoot(dir));
		}
		if((mapPluginAdded!=CMLib.map())&&(CMLib.map()!=null))
		{
			mapPluginAdded=CMLib.map();
			CMVFSDir dir=vvfs.fetchSubDir("/resources/map", false);
			if(dir!=null) vvfs.delete(dir);
			dir=vvfs.fetchSubDir("/resources", true);
			vvfs.add(CMLib.map().getMapRoot(dir));
		}
		return vvfs;
	}

	private static final String[] parsePathParts(String absolutePath)
	{
		absolutePath=vfsifyFilename(absolutePath);
		String name=absolutePath;
		int x=absolutePath.lastIndexOf('/');
		String path="";
		if(x>=0)
		{
			path=absolutePath.substring(0,x);
			name=absolutePath.substring(x+1);
		}
		return new String[]{absolutePath,path,name};
	}
	
	public static final String vfsifyFilename(String filename)
	{
		filename=filename.trim();
		if(filename.startsWith("::"))
			filename=filename.substring(2);
		if(filename.startsWith("//"))
			filename=filename.substring(2);
		if((filename.length()>3)
		&&(Character.isLetter(filename.charAt(0))
		&&(filename.charAt(1)==':')))
			filename=filename.substring(2);
		while(filename.startsWith("/")) filename=filename.substring(1);
		while(filename.startsWith("\\")) filename=filename.substring(1);
		while(filename.endsWith("/"))
			filename=filename.substring(0,filename.length()-1);
		while(filename.endsWith("\\"))
			filename=filename.substring(0,filename.length()-1);
		return filename.replace(pathSeparator,'/');
	}

	private final StringBuffer saveBufNormalize(final StringBuffer myRsc)
	{
		for(int i=0;i<myRsc.length();i++)
			if(myRsc.charAt(i)=='\n')
			{
				for(i=myRsc.length()-1;i>=0;i--)
					if(myRsc.charAt(i)=='\r')
						myRsc.deleteCharAt(i);
				return myRsc;
			}
		for(int i=0;i<myRsc.length();i++)
			if(myRsc.charAt(i)=='\r')
				myRsc.setCharAt(i,'\n');
		return myRsc;
	}

	public static final CMFile[] getFileList(final String parse, final MOB user, final boolean recurse, final boolean expandDirs)
	{
		final boolean demandLocal=parse.trim().startsWith("//");
		final boolean demandVFS=parse.trim().startsWith("::");
		CMFile dirTest=new CMFile(parse,user);
		if((dirTest.exists())&&(dirTest.isDirectory())&&(dirTest.canRead())&&(!recurse))
		{ return expandDirs?dirTest.listFiles():new CMFile[]{dirTest};}
		final String vsPath=vfsifyFilename(parse);
		String fixedName=vsPath;
		int x=vsPath.lastIndexOf('/');
		String fixedPath="";
		if(x>=0)
		{
			fixedPath=vsPath.substring(0,x);
			fixedName=vsPath.substring(x+1);
		}
		final CMFile dir=new CMFile((demandLocal?"//":demandVFS?"::":"")+fixedPath,user);
		if((!dir.exists())||(!dir.isDirectory())||(!dir.canRead()))
			return null;
		final Vector<CMFile> set=new Vector<CMFile>();
		CMFile[] cset=dir.listFiles();
		fixedName=fixedName.toUpperCase();
		for(int c=0;c<cset.length;c++)
		{
			if((recurse)&&(cset[c].isDirectory())&&(cset[c].canRead()))
			{
				final CMFile[] CF2=getFileList(cset[c].getVFSPathAndName()+"/"+fixedName,user,true,expandDirs);
				for(int cf2=0;cf2<CF2.length;cf2++)
					set.addElement(CF2[cf2]);
			}
			final String name=cset[c].getName().toUpperCase();
			boolean ismatch=true;
			if((!name.equalsIgnoreCase(fixedName))
				&&(fixedName.length()>0))
			for(int f=0,n=0;f<fixedName.length();f++,n++)
				if(fixedName.charAt(f)=='?')
				{
					if(n>=name.length()){ ismatch=false; break; }
				}
				else
				if(fixedName.charAt(f)=='*')
				{
					if(f==fixedName.length()-1) break;
					int endOfMatchStr=fixedName.indexOf('*',f+1);
					if(endOfMatchStr<0) endOfMatchStr=fixedName.indexOf('?',f+1);
					int mbEnd = fixedName.length();
					if(endOfMatchStr>f)
						mbEnd = endOfMatchStr;
					String matchBuf = fixedName.substring(f+1,mbEnd);
					int found = name.indexOf(matchBuf,n);
					if(found < 0)
					{
						ismatch=false;
						break;
					}
					else
					{
						n=found + matchBuf.length() - 1;
						f+=matchBuf.length();
					}
				}
				else
				if((n>=name.length())
				||(fixedName.charAt(f)!=name.charAt(n))
				||((f==fixedName.length()-1)&&(n<name.length()-1)))
				{ 
					ismatch=false; 
					break; 
				}
			if(ismatch) set.addElement(cset[c]);
		}
		if(set.size()==1)
		{
			dirTest=set.firstElement();
			if((dirTest.exists())&&(dirTest.isDirectory())&&(dirTest.canRead())&&(!recurse))
			{ return expandDirs?dirTest.listFiles():new CMFile[]{dirTest};}
		}
		cset=new CMFile[set.size()];
		for(int s=0;s<set.size();s++)
			cset[s]=set.elementAt(s);
		return cset;
	}
	
	//Compares two abstract pathnames lexicographically.
	@Override
	public int	compareTo(File pathname) 
	{
		if(pathname instanceof CMFile)
			return ((CMFile)pathname).getAbsolutePath().compareTo(getAbsolutePath());
		else
		{
			if(this.localFile!=null)
				return this.localFile.compareTo(pathname);
		}
		return pathname.compareTo(this);
	}
	//Atomically creates a new, empty file named by this abstract pathname if and only if a file with this name does not yet exist.
	@Override
	public boolean	createNewFile() throws IOException
	{
		if(isVFSFile())
		{
			if(!exists())
				return this.saveRaw(new byte[0]);
			else
				return false;
		}
		else
		if(this.localFile!=null)
			return this.localFile.createNewFile();
		return false;
	}
//	          Requests that the file or directory denoted by this abstract pathname be deleted when the virtual machine terminates.
	@Override
	public void	deleteOnExit() {
		if((this.localFile!=null)&&(!this.isVFSFile()))
			this.localFile.deleteOnExit();
	}
//	          Tests this abstract pathname for equality with the given object.
	@Override
	public boolean	equals(Object obj) 
	{
		if(obj instanceof CMFile)
			return ((CMFile)obj).getAbsolutePath().equalsIgnoreCase(getAbsolutePath());
		else
		if(obj instanceof File)
		{
			if(this.localFile!=null)
				return this.localFile.equals(obj);
		}
		return obj==this;
	}
//	          Returns the absolute form of this abstract pathname.
	@Override
	public File	getAbsoluteFile() { return this; }
//	          Returns the canonical form of this abstract pathname.
	@Override
	public File	getCanonicalFile() { return this; } 
//	          Returns the number of unallocated bytes in the partition named by this abstract path name.
	@Override
	public long	getFreeSpace() 
	{
		if(this.localFile!=null)
			return this.localFile.getFreeSpace();
		return 65536;
	}
//	          Returns the pathname string of this abstract pathname's parent, or null if this pathname does not name a parent directory.
	@Override
	public String	getParent()
	{
		return this.getParentFile().getAbsolutePath();
	}
//	          Converts this abstract pathname into a pathname string.
	@Override
	public String	getPath() { return this.getAbsolutePath(); }
//	          Returns the size of the partition named by this abstract pathname.
	@Override
	public long	getTotalSpace() 
	{
		if(this.localFile!=null)
			return this.localFile.getTotalSpace();
		return 65536;
	}
//	          Returns the number of bytes available to this virtual machine on the partition named by this abstract pathname.
	@Override
	public long	getUsableSpace() 
	{
		if(this.localFile!=null)
			return this.localFile.getUsableSpace();
		return 65536;
	}
//	          Computes a hash code for this abstract pathname.
	@Override
	public int	hashCode() { return this.getAbsolutePath().hashCode(); }
//	          Tests whether this abstract pathname is absolute.
	@Override
	public boolean	isAbsolute() { return true; } 
//	          Tests whether the file named by this abstract pathname is a hidden file.
	@Override
	public boolean	isHidden() { return false; }
//	          Returns the length of the file denoted by this abstract pathname.
	@Override
	public long	length()
	{
		if(!this.exists())
			return 0;
		if(this.isDirectory())
			return 0;
		if(this.isLocalFile()&&(this.localFile!=null))
			return this.localFile.length();
		final byte[] buf=this.raw();
		if(buf==null) return 0;
		return buf.length;
	}
//	          Returns an array of strings naming the files and directories in the directory denoted by this abstract pathname that satisfy the specified filter.
	@Override
	public String[]	list(FilenameFilter filter) 
	{
		if(filter==null) return this.list();
		List<String> filteredList=new Vector<String>();
		for(CMFile f : this.listFiles())
			if(filter.accept(f.getParentFile(), f.getName()))
				filteredList.add(f.getName());
		return filteredList.toArray(new String[0]);
	}
//	          Returns an array of abstract pathnames denoting the files and directories in the directory denoted by this abstract pathname that satisfy the specified filter.
	@Override
	public File[]	listFiles(FileFilter filter)
	{
		if(filter==null) return this.listFiles();
		List<CMFile> filteredList=new Vector<CMFile>();
		for(CMFile f : this.listFiles())
			if(filter.accept(f))
				filteredList.add(f);
		return filteredList.toArray(new CMFile[0]);
	}
//	          Returns an array of abstract pathnames denoting the files and directories in the directory denoted by this abstract pathname that satisfy the specified filter.
	@Override
	public File[]	listFiles(FilenameFilter filter) 
	{
		if(filter==null) return this.listFiles();
		List<CMFile> filteredList=new Vector<CMFile>();
		for(CMFile f : this.listFiles())
			if(filter.accept(f.getParentFile(),f.getName()))
				filteredList.add(f);
		return filteredList.toArray(new CMFile[0]);
	}
//	          Creates the directory named by this abstract pathname, including any necessary but nonexistent parent directories.
	@Override
	public boolean	mkdirs()
	{
		if((this.localFile!=null)&&(this.canLocalEquiv()))
			return this.localFile.mkdirs();
		return true;
	}
//	          Renames the file denoted by this abstract pathname.
	@Override
	public boolean	renameTo(File dest)
	{
		if(dest.exists())
			return false;
		if(dest instanceof CMFile)
		{
			CMFile cmDest=(CMFile)dest;
			if((this.canVFSEquiv())&&(cmDest.canVFSEquiv()))
			{
				if(!this.isDirectory())
				{
					if(!cmDest.saveRaw(this.raw()))
						return false;
					this.deleteVFS();
				}
			}
			if(this.canLocalEquiv()&&(cmDest.canLocalEquiv()))
			{
				if((this.localFile!=null)&&(this.localFile.exists()))
					return this.localFile.renameTo(new File(cmDest.getLocalPathAndName()));
				return false;
			}
		}
		else
		if(this.canLocalEquiv())
		{
			if((this.localFile!=null)&&(this.localFile.exists()))
				return this.localFile.renameTo(dest);
			return false;
		}
		return false;
	}
//	          A convenience method to set the owner's execute permission for this abstract pathname.
	@Override
	public boolean	setExecutable(boolean executable)  { return false; }
//	          Sets the owner's or everybody's execute permission for this abstract pathname.
	@Override
	public boolean	setExecutable(boolean executable, boolean ownerOnly)  { return false; }
//	          Sets the last-modified time of the file or directory named by this abstract pathname.
	@Override
	public boolean	setLastModified(long time)  { return false; }
//	          A convenience method to set the owner's read permission for this abstract pathname.
	@Override
	public boolean	setReadable(boolean readable) { return false; }
//	          Sets the owner's or everybody's read permission for this abstract pathname.
	@Override
	public boolean	setReadable(boolean readable, boolean ownerOnly) { return false; }
//	          Marks the file or directory named by this abstract pathname so that only read operations are allowed.
	@Override
	public boolean	setReadOnly() { return false; }
//	          A convenience method to set the owner's write permission for this abstract pathname.
	@Override
	public boolean	setWritable(boolean writable) { return false; }
//	          Sets the owner's or everybody's write permission for this abstract pathname.
	@Override
	public boolean	setWritable(boolean writable, boolean ownerOnly) { return false; } 
//	          Returns the pathname string of this abstract pathname.
	@Override
	public String	toString() { return this.getAbsolutePath(); }
	
	/**
	 * FileManager handler for CMFile, used by MiniWebServer
	 * @author Bo Zimmerman
	 */
	public static class CMFileManager implements FileManager 
	{
		@Override public char getFileSeparator() { 
			return '/';
		}
		@Override public File createFileFromPath(String localPath) {
			return new CMFile(localPath,null);
		}
		@Override public File createFileFromPath(File parent, String localPath) {
			return new CMFile(parent.getAbsolutePath()+'/'+localPath,null);
		}
		@Override public byte[] readFile(File file) throws IOException, FileNotFoundException {
			return ((CMFile)file).raw();
		}
		@Override public InputStream getFileStream(File file) throws IOException, FileNotFoundException {
			return ((CMFile)file).getRawStream();
		}
		@Override
		public RandomAccessFile getRandomAccessFile(File file) throws IOException, FileNotFoundException {
			return new RandomAccessFile(new File(((CMFile)file).getLocalPathAndName()),"r");
		}
		@Override
		public boolean supportsRandomAccess(File file) { 
			return ((CMFile)file).isLocalFile();
		}
	}
}
